﻿using UnityEngine;
using System;

namespace MacGruber
{
    public class AdjustableChair : MVRScript
    {
		private JSONStorableFloat myBaseExt;
		private JSONStorableFloat myBaseJoint;
		private JSONStorableFloat myBackJoint;
		private JSONStorableFloat myBackExt;
		private JSONStorableFloat myHeadJoint;
		private JSONStorableFloat myHeadExt;
		private JSONStorableFloat myHipExt;
		private JSONStorableFloat myThighJoint;
		private JSONStorableFloat myHipJoint;
		private JSONStorableFloat myThighExt;
		private JSONStorableFloat myThighSpreader;
		private JSONStorableFloat myKneeJoint;
		private JSONStorableFloat myShankJoint;
		private JSONStorableFloat myFootExt;

		private JSONStorableFloat myShoulderJoint;
		private JSONStorableFloat myArmExt;
		private JSONStorableFloat myArmSpreader;
		private JSONStorableFloat myArmJoint;
		private JSONStorableFloat myElbowJoint;
		private JSONStorableFloat myHandExt;
		private JSONStorableBool myHandSpreader;

		private Transform myRoot;

		private Transform BaseExt;
		private Transform BaseJoint;
		private Transform BackJoint;
		private Transform BackExt;
		private Transform HeadJoint;
		private Transform HeadExt;

		private Transform HipExtL;
		private Transform HipJointL;
		private Transform ThighJointL;
		private Transform ThighExtL;
		private Transform ThighSpreaderL;
		private Transform KneeJointL;
		private Transform ShankJointL;
		private Transform FootExtL;

		private Transform HipExtR;
		private Transform HipJointR;
		private Transform ThighJointR;
		private Transform ThighExtR;
		private Transform ThighSpreaderR;
		private Transform KneeJointR;
		private Transform ShankJointR;
		private Transform FootExtR;

		private Transform ShoulderJointL;
		private Transform ArmExtL;
		private Transform ArmSpreaderL;
		private Transform ArmJointL;
		private Transform ElbowJointL;
		private Transform HandExtL;
		private Transform HandSpreaderL;
		
		private Transform ShoulderJointR;
		private Transform ArmExtR;
		private Transform ArmSpreaderR;
		private Transform ArmJointR;
		private Transform ElbowJointR;
		private Transform HandExtR;
		private Transform HandSpreaderR;

		public override void Init()
		{
			// main
			myBaseExt = SetupSliderFloat(this, "Base Extender", 20, 0, 40, false);
			myBaseJoint = SetupSliderFloat(this, "Base Joint", 0, -60, 60, false);
			myBackJoint = SetupSliderFloat(this, "Back Joint", 0, 0, 60, true);
			myBackExt = SetupSliderFloat(this, "Back Extender", 3, 0, 20, true);
			myHeadJoint = SetupSliderFloat(this, "Head Joint", 0, 0, 90, true);
			myHeadExt = SetupSliderFloat(this, "Head Extender", 3, 0, 20, true);
			
			myBaseExt.setCallbackFunction += (float v) => {
				if (BaseExt != null) BaseExt.localPosition = new Vector3(0, 0, v/100);
			};		
			myBaseJoint.setCallbackFunction += (float v) => {
				if (BaseJoint != null) BaseJoint.localEulerAngles = new Vector3(v, 0, 0);
			};
			myBackJoint.setCallbackFunction += (float v) => {
				if (BackJoint != null) BackJoint.localEulerAngles = new Vector3(v, 0, 0);
			};			
			myBackExt.setCallbackFunction += (float v) => {
				if (BackExt != null) BackExt.localPosition = new Vector3(0, v/100, 0);
			};
			myHeadJoint.setCallbackFunction += (float v) => {
				if (HeadJoint != null) HeadJoint.localEulerAngles = new Vector3(v, 0, 0);
			};
			myHeadExt.setCallbackFunction += (float v) => {
				if (HeadExt != null) HeadExt.localPosition = new Vector3(0, v/100, 0);
			};	
			
			// arms
			myShoulderJoint = SetupSliderFloat(this, "Shoulder Joint", 0, -70, 70, true);
			myArmExt = SetupSliderFloat(this, "Arm Extender", 5, 0, 18, true);
			myArmSpreader = SetupSliderFloat(this, "Arm Spreader", 0, -60, 60, true);
			myArmJoint = SetupSliderFloat(this, "Arm Joint", 0, -100, 100, true);
			myElbowJoint = SetupSliderFloat(this, "Elbow Joint", 0, -45, 90, true);
			myHandExt = SetupSliderFloat(this, "Hand Extender", 5, 0, 20, true);
			myHandSpreader = SetupToggle(this, "Hand Spreader", false, true);
			
			myShoulderJoint.setCallbackFunction += (float v) => {
				if (ShoulderJointL != null) ShoulderJointL.localEulerAngles = new Vector3(0, 0, -v);
				if (ShoulderJointR != null) ShoulderJointR.localEulerAngles = new Vector3(0, 0, 180+v);
			};
			myArmExt.setCallbackFunction += (float v) => {
				if (ArmExtL != null) ArmExtL.localPosition = new Vector3(-v/100, 0, 0);
				if (ArmExtR != null) ArmExtR.localPosition = new Vector3(-v/100, 0, 0);
			};
			myArmSpreader.setCallbackFunction += (float v) => {
				if (ArmSpreaderL != null) ArmSpreaderL.localEulerAngles = new Vector3(0, 90-v, 90);
				if (ArmSpreaderR != null) ArmSpreaderR.localEulerAngles = new Vector3(0, 90-v, 90);
			};			
			myArmJoint.setCallbackFunction += (float v) => {
				if (ArmJointL != null) ArmJointL.localEulerAngles = new Vector3(0, 0, -v);
				if (ArmJointR != null) ArmJointR.localEulerAngles = new Vector3(180, 0, -v);
			};
			myElbowJoint.setCallbackFunction += (float v) => {
				if (ElbowJointL != null) ElbowJointL.localEulerAngles = new Vector3(0, v, 0);
				if (ElbowJointR != null) ElbowJointR.localEulerAngles = new Vector3(180, -v, 0);
			};
			myHandExt.setCallbackFunction += (float v) => {
				if (HandExtL != null) HandExtL.localPosition = new Vector3(-v/100, 0, 0);
				if (HandExtR != null) HandExtR.localPosition = new Vector3(-v/100, 0, 0);
			};
			myHandSpreader.setCallbackFunction += (bool v) => {
				if (HandSpreaderL != null) HandSpreaderL.localEulerAngles = new Vector3(v ? 180 : 0, 90, 90);
				if (HandSpreaderR != null) HandSpreaderR.localEulerAngles = new Vector3(v ? 0 : 180, 90, 90);
			};
			
			// legs
			myHipExt = SetupSliderFloat(this, "Hip Extender", 2, 0, 6, false);
			myHipJoint = SetupSliderFloat(this, "Hip Joint", 0, 0, 90, false);
			myThighJoint = SetupSliderFloat(this, "Thigh Joint", 0, 0, 80, false);
			myThighExt = SetupSliderFloat(this, "Thigh Extender", 6, 0, 14, false);
			myThighSpreader = SetupSliderFloat(this, "Thigh Spreader", 0, 0, 90, false);
			myKneeJoint = SetupSliderFloat(this, "Knee Joint", 0, 0, 90, false);
			myShankJoint = SetupSliderFloat(this, "Shank Joint", 0, 0, 45, false);
			myFootExt = SetupSliderFloat(this, "Foot Extender", 2, 0, 17, false);
			
			myHipExt.setCallbackFunction += (float v) => {
				if (HipExtL != null) HipExtL.localPosition = new Vector3(-v/100, 0, 0);
				if (HipExtR != null) HipExtR.localPosition = new Vector3(v/100, 0, 0);
			};
			myHipJoint.setCallbackFunction += (float v) => {
				if (HipJointL != null) HipJointL.localEulerAngles = new Vector3(0, 180, 90+v);
				if (HipJointR != null) HipJointR.localEulerAngles = new Vector3(0, 0, 90+v);
			};
			myThighJoint.setCallbackFunction += (float v) => {
				if (ThighJointL != null) ThighJointL.localEulerAngles = new Vector3(90, 0, -90+v);
				if (ThighJointR != null) ThighJointR.localEulerAngles = new Vector3(-90, 0, -90+v);
			};			
			myThighExt.setCallbackFunction += (float v) => {
				if (ThighExtL != null) ThighExtL.localPosition = new Vector3(0, -v/100, 0);
				if (ThighExtR != null) ThighExtR.localPosition = new Vector3(0, -v/100, 0);
			};
			myThighSpreader.setCallbackFunction += (float v) => {
				if (ThighSpreaderL != null) ThighSpreaderL.localEulerAngles = new Vector3(90+v, 0, 0);
				if (ThighSpreaderR != null) ThighSpreaderR.localEulerAngles = new Vector3(-90+v, 0, 180);
			};
			myKneeJoint.setCallbackFunction += (float v) => {
				if (KneeJointL != null) KneeJointL.localEulerAngles = new Vector3(0, 90, 90+v);
				if (KneeJointR != null) KneeJointR.localEulerAngles = new Vector3(0, 90, 90+v);
			};
			myShankJoint.setCallbackFunction += (float v) => {
				if (ShankJointL != null) ShankJointL.localEulerAngles = new Vector3(0, 90+v, 180);
				if (ShankJointR != null) ShankJointR.localEulerAngles = new Vector3(0, 90-v, 180);
			};
			myFootExt.setCallbackFunction += (float v) => {
				if (FootExtL != null) FootExtL.localPosition = new Vector3(0, 0, -v/100);
				if (FootExtR != null) FootExtR.localPosition = new Vector3(0, 0, -v/100);
			};
		}

		private void Update()
		{
			try {
				if (myRoot != null)
					return;
				myRoot = containingAtom.reParentObject.Find("object/rescaleObject/AdjustableChair(Clone)");
				if (myRoot != null)
					UpdateTransforms();
			}
			catch (Exception e)
			{
				SuperController.LogError(e.ToString());
			}
		}

		private void UpdateTransforms()
		{
			// find transforms
			BaseExt = myRoot.Find("BaseExt");
			BaseJoint = BaseExt.Find("BaseJoint");
			BackJoint = BaseJoint.Find("BackJoint");
			BackExt = BackJoint.Find("BackExt");
			HeadJoint = BackExt.Find("HeadJoint");
			HeadExt = HeadJoint.Find("HeadExt");
			
			ShoulderJointL = BackExt.Find("ShoulderJointL");
			ArmExtL = ShoulderJointL.Find("ArmExtL");
			ArmSpreaderL = ArmExtL.Find("ArmSpreaderL");
			ArmJointL = ArmExtL.Find("ArmJointL");
			ElbowJointL = ArmJointL.Find("ElbowJointL");
			HandExtL = ElbowJointL.Find("HandExtL");
			HandSpreaderL = HandExtL.Find("HandSpreaderL");
						
			ShoulderJointR = BackExt.Find("ShoulderJointR");
			ArmExtR = ShoulderJointR.Find("ArmExtR");
			ArmSpreaderR = ArmExtR.Find("ArmSpreaderR");
			ArmJointR = ArmExtR.Find("ArmJointR");
			ElbowJointR = ArmJointR.Find("ElbowJointR");
			HandExtR = ElbowJointR.Find("HandExtR");
			HandSpreaderR = HandExtR.Find("HandSpreaderR");
			
			HipExtL = BaseJoint.Find("HipExtL");
			HipJointL = HipExtL.Find("HipJointL");
			ThighJointL = HipJointL.Find("ThighJointL");
			ThighExtL = ThighJointL.Find("ThighExtL");
			ThighSpreaderL = ThighExtL.Find("ThighSpreaderL");
			KneeJointL = ThighExtL.Find("KneeJointL");
			ShankJointL = KneeJointL.Find("ShankJointL");
			FootExtL = ShankJointL.Find("FootExtL");

			HipExtR = BaseJoint.Find("HipExtR");
			HipJointR = HipExtR.Find("HipJointR");
			ThighJointR = HipJointR.Find("ThighJointR");
			ThighExtR = ThighJointR.Find("ThighExtR");
			ThighSpreaderR = ThighExtR.Find("ThighSpreaderR");
			KneeJointR = ThighExtR.Find("KneeJointR");
			ShankJointR = KneeJointR.Find("ShankJointR");
			FootExtR = ShankJointR.Find("FootExtR");
			
			// force apply transforms
			myBaseExt.setCallbackFunction(myBaseExt.val);
			myBaseJoint.setCallbackFunction(myBaseJoint.val);
			myBackJoint.setCallbackFunction(myBackJoint.val);
			myBackExt.setCallbackFunction(myBackExt.val);
			myHeadJoint.setCallbackFunction(myHeadJoint.val);
			myHeadExt.setCallbackFunction(myHeadExt.val);
			
			myShoulderJoint.setCallbackFunction(myShoulderJoint.val);
			myArmExt.setCallbackFunction(myArmExt.val);
			myArmSpreader.setCallbackFunction(myArmSpreader.val);
			myArmJoint.setCallbackFunction(myArmJoint.val);
			myElbowJoint.setCallbackFunction(myElbowJoint.val);
			myHandExt.setCallbackFunction(myHandExt.val);
			myHandSpreader.setCallbackFunction(myHandSpreader.val);
			
			myHipExt.setCallbackFunction(myHipExt.val);
			myHipJoint.setCallbackFunction(myHipJoint.val);
			myThighJoint.setCallbackFunction(myThighJoint.val);
			myThighExt.setCallbackFunction(myThighExt.val);
			myThighSpreader.setCallbackFunction(myThighSpreader.val);
			myKneeJoint.setCallbackFunction(myKneeJoint.val);
			myShankJoint.setCallbackFunction(myShankJoint.val);
			myFootExt.setCallbackFunction(myFootExt.val);
		}

		private static JSONStorableFloat SetupSliderFloat(MVRScript script, string label, float defaultValue, float minValue, float maxValue, bool rightSide)
		{
			JSONStorableFloat storable = new JSONStorableFloat(label, defaultValue, minValue, maxValue, true, true);
			storable.storeType = JSONStorableParam.StoreType.Full;
			UIDynamicSlider slider = script.CreateSlider(storable, rightSide);
			script.RegisterFloat(storable);
			return storable;
		}
		
		private static JSONStorableBool SetupToggle(MVRScript script, string label, bool defaultValue, bool rightSide)
		{
			JSONStorableBool storable = new JSONStorableBool(label, defaultValue);
			storable.storeType = JSONStorableParam.StoreType.Full;
			script.CreateToggle(storable, rightSide);
			script.RegisterBool(storable);
			return storable;
		}
    }
}
